#----------------------------------------------------------------------
# This file was generated by C:\PROGRA~1\PROGRA~1\Python22\Scripts\img2py
#
from wxPython.wx import wxImageFromStream, wxBitmapFromImage
import io, zlib


def getHSplitData():
    return zlib.decompress(
'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2\n \xcc\xc1\x06\
$\x8b\xab\xaa\xbe\x00)\x96b\'\xcf\x10\x0e \xa8\xe1H\xe9\x00\xf2\xdd<]\x1cC*\
\xe6\xbc\xbd\xbe\x91\x97A\x81\x87\xf9 \x7f\xd2\x86\x9eyg\x17,\xe3qZTY\xae\
\x99\x13;\x9bAL\xde\x98q\xd5\xaa\xd83g\x1e\xef\xbb+\xf1\xf6\xdac\x0e\x01\x05\
\x87\x06\x06F&\x16(\xe3\xe0\xf3\xef5\xcc\xb6G9\xec\xab\x8fWs\x02Md\xf0t\xf5s\
Y\xe7\x94\xd0\x04\x00GI*+' )

def getHSplitBitmap():
    return wxBitmapFromImage(getHSplitImage())

def getHSplitImage():
    stream = io.StringIO(getHSplitData())
    return wxImageFromStream(stream)

#----------------------------------------------------------------------
def getVSplitData():
    return zlib.decompress(
'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2\n \xcc\xc1\x06\
$\x8b\xab\xaa\xbe\x00)\x96b\'\xcf\x10\x0e \xa8\xe1H\xe9\x00\xf2\xdd=]\x1cC*\
\xe6\xbc\xbd\xb6\x91\xf7\x80\x03\xcfa\x87\xba\t\x07mv\x99\x9f\xde~\xb3\x81wk\
\x03_\xde\x1a\xe6\x9a\x8eJ\xc6\xb2p\xcd\xf9\x92rK\x7fNg`db\xe1X\xc0\xcc\xf7\
\xc9\xfc\x8a\xbe\xc0\x02G!\x05\x87\x06\x06\xc6\x92\x06;\x0f\xbd\x9f\x878\xed\
o\xbd\xdde\x0e4\x92\xc1\xd3\xd5\xcfe\x9dSB\x13\x00\xb5\x8e+b' )

def getVSplitBitmap():
    return wxBitmapFromImage(getVSplitImage())

def getVSplitImage():
    stream = io.StringIO(getVSplitData())
    return wxImageFromStream(stream)

#----------------------------------------------------------------------
def getHVSplitData():
    return zlib.decompress(
'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2\n \xcc\xc1\x06\
$\x8b\xab\xaa\xbe\x00)\x96b\'\xcf\x10\x0e \xa8\xe1H\xe9\x00\xf2\xfd=]\x1cC*\
\xe6\xbc\xbdf\xc8w\xc0\x80\x83\xe5\xe0\xd6\xa9\xed\x1fK\xea\xd2\xaekv*\xc5n.\
\x11iw3\xd9\xd0\xe0\xbcA\xae\xe1\x9d\x1c\xfb\xeaWW\xeb\xe7n<\xber\xbew\x03\
\x03#\x13\xcb\x11\xc6\xee\x0f\xa6=\xf9\x0e\r\xfc\x9d .G\x07\xf3#\xfe\x93\xac\
\t\xeb\xf9\xed\xdf\xaao\xce\x06\x1a\xcd\xe0\xe9\xea\xe7\xb2\xce)\xa1\t\x00\
\x7f\xcc/\x9d' )

def getHVSplitBitmap():
    return wxBitmapFromImage(getHVSplitImage())

def getHVSplitImage():
    stream = io.StringIO(getHVSplitData())
    return wxImageFromStream(stream)

