# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Python Computer Graphics Kit.
#
# The Initial Developer of the Original Code is Matthias Baas.
# Portions created by the Initial Developer are Copyright (C) 2004
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

"""This module contains the MBReader base class to parse Maya binary files (~IFF files).
"""

from . import mayaiff

class MBReader(mayaiff.IFFReader):
    """Read Maya binary files and call an appropriate handler for each chunk.
    
    This is the base class for a .mb reader class. Derived classes can implement
    the chunk handler methods onBeginGroup(), onEndGroup() and onDataChunk().
    These handlers receive a Chunk (or GroupChunk) object as input that
    contains information about the current chunk and that can also be used
    to read the actual chunk data.
    """
    
    def __init__(self):
        mayaiff.IFFReader.__init__(self, "Maya")
            
            
if __name__=="__main__":
    
    import sys
    
    class MBDumper(MBReader):
        def onBeginGroup(self, chunk):
            print(("GRP BEGIN %s"%chunk))
        
        def onEndGroup(self, chunk):
            print(("GRP END %s"%chunk))
        
        def onDataChunk(self, chunk):
            print(("CHUNK %s"%chunk))
            
    rd = MBDumper()
    rd.read(open(sys.argv[1], "rb"))
    