# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the License); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Python Computer Graphics Kit.
#
# The Initial Developer of the Original Code is Matthias Baas.
# Portions created by the Initial Developer are Copyright (C) 2010
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the GPL), or
# the GNU Lesser General Public License Version 2.1 or later (the LGPL),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# Versions this module is based on
LIBAVCODEC_VERSION_INT = 3439104
LIBAVCODEC_VERSION_MAJOR = 52
LIBAVCODEC_VERSION_MICRO = 0
LIBAVCODEC_VERSION_MINOR = 122

LIBAVFORMAT_VERSION_INT = 3436032
LIBAVFORMAT_VERSION_MAJOR = 52
LIBAVFORMAT_VERSION_MICRO = 0
LIBAVFORMAT_VERSION_MINOR = 110

LIBAVUTIL_VERSION_INT = 3287808
LIBAVUTIL_VERSION_MAJOR = 50
LIBAVUTIL_VERSION_MICRO = 0
LIBAVUTIL_VERSION_MINOR = 43

LIBAVFILTER_VERSION_INT = 86016
LIBAVFILTER_VERSION_MAJOR = 1
LIBAVFILTER_VERSION_MICRO = 0
LIBAVFILTER_VERSION_MINOR = 80

LIBSWSCALE_VERSION_INT = 3585
LIBSWSCALE_VERSION_MAJOR = 0
LIBSWSCALE_VERSION_MICRO = 1
LIBSWSCALE_VERSION_MINOR = 14

AV_NOPTS_VALUE = 0x8000000000000000

AV_TIME_BASE = 1000000

MAX_STREAMS = 20

# Error codes
AVERROR_BSF_NOT_FOUND = -1179861752
AVERROR_DECODER_NOT_FOUND = -1128613112
AVERROR_DEMUXER_NOT_FOUND = -1296385272
AVERROR_ENCODER_NOT_FOUND = -1129203192
AVERROR_EOF = -32
AVERROR_EXIT = -1414092869
AVERROR_FILTER_NOT_FOUND = -1279870712
AVERROR_INVALIDDATA = -22
AVERROR_IO = -5
AVERROR_MUXER_NOT_FOUND = -1481985528
AVERROR_NOENT = -2
AVERROR_NOFMT = -92
AVERROR_NOMEM = -12
AVERROR_NOTSUPP = -78
AVERROR_NUMEXPECTED = -33
AVERROR_OPTION_NOT_FOUND = -1414549496
AVERROR_PATCHWELCOME = -1163346256
AVERROR_PROTOCOL_NOT_FOUND = -1330794744
AVERROR_STREAM_NOT_FOUND = -1381258232
AVERROR_UNKNOWN = -22

# CodecType enum values
CODEC_TYPE_ATTACHMENT = 4
CODEC_TYPE_AUDIO = 1
CODEC_TYPE_DATA = 2
CODEC_TYPE_NB = 5
CODEC_TYPE_SUBTITLE = 3
CODEC_TYPE_UNKNOWN = -1
CODEC_TYPE_VIDEO = 0

# Codec flags
CODEC_FLAG_4MV = 4
CODEC_FLAG_AC_PRED = 16777216
CODEC_FLAG_ALT_SCAN = 1048576
CODEC_FLAG_BITEXACT = 8388608
CODEC_FLAG_CBP_RD = 67108864
CODEC_FLAG_CLOSED_GOP = -2147483648
CODEC_FLAG_EMU_EDGE = 16384
CODEC_FLAG_EXTERN_HUFF = 4096
CODEC_FLAG_GLOBAL_HEADER = 4194304
CODEC_FLAG_GMC = 32
CODEC_FLAG_GRAY = 8192
CODEC_FLAG_H263P_AIV = 8
CODEC_FLAG_H263P_SLICE_STRUCT = 268435456
CODEC_FLAG_H263P_UMV = 33554432
CODEC_FLAG_INPUT_PRESERVED = 256
CODEC_FLAG_INTERLACED_DCT = 262144
CODEC_FLAG_INTERLACED_ME = 536870912
CODEC_FLAG_LOOP_FILTER = 2048
CODEC_FLAG_LOW_DELAY = 524288
CODEC_FLAG_MV0 = 64
CODEC_FLAG_NORMALIZE_AQP = 131072
CODEC_FLAG_OBMC = 1
CODEC_FLAG_PART = 128
CODEC_FLAG_PASS1 = 512
CODEC_FLAG_PASS2 = 1024
CODEC_FLAG_PSNR = 32768
CODEC_FLAG_QPEL = 16
CODEC_FLAG_QP_RD = 134217728
CODEC_FLAG_QSCALE = 2
CODEC_FLAG_SVCD_SCAN_OFFSET = 1073741824
CODEC_FLAG_TRUNCATED = 65536

# Flags for the "flags" field in AVOutputFormat/AVInputFormat
AVFMT_FLAG_CUSTOM_IO = 128
AVFMT_FLAG_GENPTS = 1
AVFMT_FLAG_IGNDTS = 8
AVFMT_FLAG_IGNIDX = 2
AVFMT_FLAG_KEEP_SIDE_DATA = 262144
AVFMT_FLAG_MP4A_LATM = 32768
AVFMT_FLAG_NOFILLIN = 16
AVFMT_FLAG_NONBLOCK = 4
AVFMT_FLAG_NOPARSE = 32
AVFMT_FLAG_PRIV_OPT = 131072
AVFMT_FLAG_RTP_HINT = 64
AVFMT_FLAG_SORT_DTS = 65536
AVFMT_GENERIC_INDEX = 256
AVFMT_GLOBALHEADER = 64
AVFMT_INFINITEOUTPUTLOOP = 0
AVFMT_NEEDNUMBER = 2
AVFMT_NOBINSEARCH = 8192
AVFMT_NODIMENSIONS = 2048
AVFMT_NOFILE = 1
AVFMT_NOGENSEARCH = 16384
AVFMT_NOOUTPUTLOOP = -1
AVFMT_NOSTREAMS = 4096
AVFMT_NOTIMESTAMPS = 128
AVFMT_RAWPICTURE = 32
AVFMT_SHOW_IDS = 8
AVFMT_TS_DISCONT = 512
AVFMT_TS_NONSTRICT = 32768
AVFMT_VARIABLE_FPS = 1024

# Flags for url_fopen()
URL_RDONLY = 0
URL_RDWR = 2
URL_WRONLY = 1

# av_seek_frame() flags
AVSEEK_FLAG_ANY = 4
AVSEEK_FLAG_BACKWARD = 1
AVSEEK_FLAG_BYTE = 2
AVSEEK_FLAG_FRAME = 8

# swscale flags
SWS_ACCURATE_RND = 262144
SWS_AREA = 32
SWS_BICUBIC = 4
SWS_BICUBLIN = 64
SWS_BILINEAR = 2
SWS_BITEXACT = 524288
SWS_CPU_CAPS_3DNOW = 1073741824
SWS_CPU_CAPS_ALTIVEC = 268435456
SWS_CPU_CAPS_BFIN = 16777216
SWS_CPU_CAPS_MMX = -2147483648
SWS_CPU_CAPS_MMX2 = 536870912
SWS_CPU_CAPS_SSE2 = 33554432
SWS_CS_DEFAULT = 5
SWS_CS_FCC = 4
SWS_CS_ITU601 = 5
SWS_CS_ITU624 = 5
SWS_CS_ITU709 = 1
SWS_CS_SMPTE170M = 5
SWS_CS_SMPTE240M = 7
SWS_DIRECT_BGR = 32768
SWS_FAST_BILINEAR = 1
SWS_FULL_CHR_H_INP = 16384
SWS_FULL_CHR_H_INT = 8192
SWS_GAUSS = 128
SWS_LANCZOS = 512
SWS_PARAM_DEFAULT = 123456
SWS_POINT = 16
SWS_PRINT_INFO = 4096
SWS_SINC = 256
SWS_SPLINE = 1024
SWS_SRC_V_CHR_DROP_MASK = 196608
SWS_SRC_V_CHR_DROP_SHIFT = 16
SWS_X = 8

