# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Python Computer Graphics Kit.
#
# The Initial Developer of the Original Code is Matthias Baas.
# Portions created by the Initial Developer are Copyright (C) 2004
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****
# $Id: icamera.py,v 1.1.1.1 2004/12/12 14:31:41 mbaas Exp $

from .iworldobject import IWorldObject

class ICamera(IWorldObject):
    """The camera protocol.

    """

    def projection(self, width, height, near, far):
        """Return the projection matrix for this camera.

        This method has to return the matrix that should be used to
        project the scene onto the screen. This matrix can either
        be an orthographic or perspective matrix.

        \param width (\c int) Viewport width
        \param height (\c int) Viewport height
        \param near (\c float) Near plane value
        \param far (\c float) Far plane value
        \return Projection matrix (\c mat4)
        """

    def viewTransformation(self):
        """Return the view transformation matrix of this camera.

        The view transformation transforms from World coordinates
        into Camera coordinates and is usually the inverse of the
        \c transform attribute. However, an implementation can
        choose to cache an inverted matrix.
        """
        
